import numpy as np
from sklearn.preprocessing import StandardScaler
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt

# La función de decisión para la MVS con funciones
# básicas radiales es
# f(x) = sum y_i a_i phi(x)^T phi(x_i) + b
# 
ya = [-1.        , -1.        , -1.        , -0.91744228, -1.       ,  -1.,
      -0.30560327, -0.49875941, -0.71869164, -0.02793147, -1.       ,   0.10944263,
       0.64545777,  0.47385327,  1.        ,  1.        ,  0.9495953,   1.,
       0.71238192,  1.        ,  0.19119284,  0.38650434,  1.        ]

b = -0.03738448

VS = np.array( [[-1.63170582, -0.65021254],
 [ 0.04078673,  0.87201439] ,
 [-0.62464726,  0.99684132] ,
 [ 0.36890459,  0.22104786] ,
 [-1.14180768,  0.71833226] ,
 [ 0.46961048, -0.18609087] ,
 [ 0.03901932,  1.72971278] ,
 [-1.6069293 ,  0.12272255] ,
 [-1.36347362,  0.72142059] ,
 [ 0.45081006,  0.03654143] ,
 [ 0.77509719, -0.42688641] ,
 [ 1.59093726,  0.06616728] ,
 [ 1.09273075, -1.09795158] ,
 [ 0.16850547, -1.16576381] ,
 [-0.67006187,  0.33265936] ,
 [ 1.60063711,  0.31514952] ,
 [-0.50188524,  0.28569641] ,
 [-0.61186775,  0.38167422] ,
 [-0.14674072, -0.9380105 ] ,
 [ 1.40242131, -0.47414003] ,
 [-0.59918549, -0.69723992] ,
 [ 0.86391891, -1.74620298] ,
 [-0.5490031 ,  0.46914851]] )


def funcion_decision( vx ) :
	gamma = 1.0
	suma = 0.0
	n = 23
	i = 0
	while i < n :
		x = VS[i,0] - vx[0]
		y = VS[i,1] - vx[1]
		suma += ya[i] * np.exp( -gamma * (x*x + y*y) )
		i += 1
	suma += b

	if suma >= 0 :
		return 1
	else :
		return 0 

D = np.loadtxt( "datos.txt" )
print( D.shape )

X = D[:,0:2]
y = D[:,2]

print( X.shape )

# Como usaremos una máquina de vectores de soporte
# reescalamos los datos a media 0 y varianza igual a 1
reescalar = StandardScaler( )
reescalar.fit( X )
X2 = reescalar.transform( X )
print( X2.shape )

# Separamos los datos en datos de entrenamiento
# y datos de prueba
#

X_entrena, X_prueba, y_entrena, y_prueba = train_test_split( X2, y, random_state=2 )

# Hacemos la con la función de decisión
k, m = X_prueba.shape

i=0
while i<k :
	v = funcion_decision( X_prueba[i] )
	print( i, y_prueba[i], v )
	i += 1

# Para graficar la superficie de decisión
from matplotlib.colors import ListedColormap
  
n = 100
vx = np.linspace( -2, 2, n )
vy = np.linspace( -2.5, 2, n )

x, y = np.meshgrid( vx, vy )
Z = np.zeros( (n,n) )

va = np.zeros( (2,) )

for i in range(n):
    for j in range(n):
        va[0]  = x[j,i]
        va[1]  = y[j,i]
        Z[j,i] = funcion_decision( va )


fig, [ax1, ax2] = plt.subplots( nrows=1, ncols=2 )

ax1.scatter( X_entrena[:,0], X_entrena[:,1], c=y_entrena )
ax1.scatter( VS[:,0], VS[:,1], c='red' )
ax1.set_xlim( -2, 2 )
ax1.set_ylim( -2.5, 2 )

ax2.set_xlim( -2, 2 )
ax2.set_ylim( -2.5, 2 )

cm = plt.cm.RdBu
cm_bright = ListedColormap(['#FF0000', '#0000FF'])

ax2.contourf( x, y, Z, cmap=cm, alpha=.8 )
ax2.scatter( X_prueba[:,0], X_prueba[:,1], c=y_prueba )


plt.show( )
